
function love.load()
	
	width, height = 480, 320 -- units, not pixels
	unit = 20
	scale = 2
--	scale = 4
	love.window.setMode( width*scale, height*scale, {resizable=true} )
	love.graphics.setDefaultFilter( 'nearest', 'nearest' )
	love.graphics.setLineStyle( "rough" )

	map = {}
	for i = 0, (width-unit)/unit do
		map[i] = {}
		for j = 0, (height-unit)/unit do
			map[i][j] = 0
		end
	end
	
--	love.mouse.setGrabbed(true)
	mouse_line = {points = {}, pressed = false, last = {x=0, y=0}}
	lines = {}
	agents = {}
	
	local line = {30, 30, 60, 80, 80, 50, 300, 10}
	table.insert(lines, line)
	create_agent (line)

	
end


function create_agent (line)
	local agent = {}
	agent.line = line
	agent.t = 0
	agent.s = 0
	agent.max_v = 20
	agent.sign = 1
--	agent.position = {x=line[1], y=line[2]}
	agent.x = line[1]
	agent.y = line[2]
	agent.n = 1
	agent.rs = 0
	
	
	table.insert (agents, agent)
end

function love.update(dt)
	for i, agent in pairs (agents) do
		agent.s = agent.s + dt*agent.max_v
		agent.t = agent.t + dt
		local line = agent.line
		local n = agent.n
		local x1, y1 = line[n], line[n+1]
		local x2, y2 = line[n+2], line[n+3]
		
		local sq_section = (x2-x1)^2+(y2-y1)^2
		local sq_agent = (agent.x-x1)^2+(agent.y-y1)^2
		agent.rs = (sq_agent/sq_section)^0.5
		if agent.rs > 1 then
			n = n + 2
			if not line[n+2] then 
				n = 1
				agent.x, agent.y = line[n], line[n+1]
			end
			agent.n = n
			x1, y1 = line[n], line[n+1]
			x2, y2 = line[n+2], line[n+3]
			sq_section = (x2-x1)^2+(y2-y1)^2
			
			local s = sq_section^0.5
			agent.vx = (x2-x1)/s*agent.max_v
			agent.vy = (y2-y1)/s*agent.max_v
		elseif not agent.vx then
			local s = sq_section^0.5
			agent.vx = (x2-x1)/s*agent.max_v
			agent.vy = (y2-y1)/s*agent.max_v
		end

		agent.x = agent.x + dt*agent.vx
		agent.y = agent.y + dt*agent.vy
	end
end

function draw_mouse_line ()
	local line = {}
	for i, point in ipairs (mouse_line.points) do
		table.insert (line, point.x/scale)
		table.insert (line, point.y/scale)
	end
	if #line >4 then
		love.graphics.setColor(1,1,0)
		love.graphics.line(line)
	end
--	table.insert (lines, line)
end

function draw_map ()
	love.graphics.setColor(.1,.1,.1)
	for i, js in pairs (map) do
		for j, value in pairs (js) do
			love.graphics.rectangle('line', i*unit, j*unit, unit, unit)
		end
	end
end

function draw_mouse_poiner ()
	love.graphics.circle('line', love.mouse.getX( )/scale, love.mouse.getY( )/scale, 0.25*unit)
end


function draw_lines ()
	local line_width = love.graphics.getLineWidth( )
	love.graphics.setLineWidth( 20 )

	love.graphics.setColor(1,1,1)
	
	for i, line in pairs (lines) do
		love.graphics.circle('fill', line[1], line[2], 20/2)
		love.graphics.line(line)
		love.graphics.circle('fill', line[#line-1], line[#line], 20/2)
	end
	love.graphics.setLineWidth( line_width )

end



function love.draw()
	
	love.graphics.scale(scale)
	
	draw_map ()

	draw_lines ()

	draw_mouse_line ()
	
	love.graphics.setColor(1,0,0)
	for i, agent in pairs (agents) do
		local s = (agent.n-1)/2+agent.rs
		local sl = 0.25
		love.graphics.circle('line', agent.x, agent.y, 0.25*unit)
		love.graphics.line(agent.x-sl*agent.vx, agent.y-sl*agent.vy, agent.x+sl*agent.vx, agent.y+sl*agent.vy)
--		love.graphics.print(string.format ("%#0.1f", s), agent.x, agent.y)
--		love.graphics.print(string.format ("%#0.2f", agent.t), agent.x, agent.y)
--		love.graphics.print(string.format ("%#0.2f", agent.s), agent.x, agent.y+15)
	end
	
	draw_mouse_poiner ()
	
	
	love.graphics.print (tostring(mouse_line.pressed))
	love.graphics.print (tostring(#mouse_line.points), 0, 15)
	love.graphics.print (tostring(#lines), 0, 30)
end


function love.mousepressed(x, y, button, istouch)
	if button == 1 then
		mouse_line.pressed = true
		mouse_line.last = {x=x, y=y}
		mouse_line.points = {}
	end
end

function love.mousemoved( x, y, dx, dy, istouch )
	local ds = 0.5*unit/scale
	if mouse_line.pressed then
		if ((x-mouse_line.last.x)^2+(y-mouse_line.last.y)^2) >= (ds)^2 then
			local point = mouse_line.last
			mouse_line.last = {x=x, y=y}
			table.insert (mouse_line.points, point)
		end
	end
end

function love.mousereleased(x, y, button)
	if button == 1 then
		mouse_line.pressed = false
		local line = {}
		for i, point in ipairs (mouse_line.points) do
			table.insert (line, point.x/scale)
			table.insert (line, point.y/scale)
		end
		table.insert (lines, line)
		create_agent (line)
		mouse_line.points = {}
	end
end

function love.keypressed(key, scancode, isrepeat)
	if key == "escape" then
		love.event.quit()
	elseif key == "kp+" then
		scale = scale*2^(1/2)
	elseif key == "kp-" then
		scale = scale/2^(1/2)
	end
end